
/******************************************************************************
* MODULE     : polynomial.hpp
* DESCRIPTION: Generic univariate polynomials
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_POLYNOMIAL_HPP
#define __MMX_POLYNOMIAL_HPP
#include "string.hpp"
#include "iterator.hpp"
namespace mmx {
class naive_multiply;
template<typename V> class dense_polynomial;
#define TMPL_DEF \
  template<typename C, typename V=dense_polynomial<naive_multiply> >
#define TMPL template<typename C, typename V>
#define Polynomial polynomial<C,V>
TMPL class polynomial;

/******************************************************************************
* Abstract specification of dense polynomials
******************************************************************************/

TMPL_DEF class polynomial_rep REP_STRUCT {};

TMPL_DEF
class polynomial {
INDIRECT_PROTO_2 (polynomial, polynomial_rep, C, V)
public:
  polynomial ();
  template<typename T> polynomial (const T& c);
  template<typename T> polynomial (const T& c, nat n);
  template<typename T> polynomial (const polynomial<T,V>& P);
  polynomial (const iterator<C>& it);
  C operator[] (nat i);
};
INDIRECT_IMPL_2 (polynomial, polynomial_rep, typename C, C, typename V, V)

TMPL int deg (const Polynomial& P);

/******************************************************************************
* Polynomial iteration
******************************************************************************/

TMPL_DEF
class polynomial_iterator_rep: public iterator_rep<C> {
  Polynomial P;  // the polynomial to traverse
  nat i;         // current position
protected:
  bool is_busy () { return ((int) i) <= deg (P); }
  void advance () { i++; }
  C current () { return P[i]; }
public:
  polynomial_iterator_rep (const Polynomial& P2): P(P2), i(0) {}
};

TMPL iterator<C>
iterate (const Polynomial& P) {
  return iterator<C> (new polynomial_iterator_rep<C,V> (P));
}

/******************************************************************************
* Hashing and generic output routines for dense polynomials
******************************************************************************/

TMPL nat
hash (const Polynomial& p) {
  register nat i, h= 642531, n= dim (p);
  for (i=0; i<n; i++)
    h= (h<<1) ^ (h<<5) ^ (h>>27) ^ hash (p[i]);
  return h;
}

TMPL inline generic var (const Polynomial& P) {
  (void) P; return "x"; }

TMPL generic
flatten (const Polynomial& P, const generic& v) {
  int i, len= deg(P);
  generic s= 0;
  for (i=len; i>=0; i--)
    s= s + flatten (P[i]) * pow (v, i);
  return s;
}

TMPL generic
flatten (const Polynomial& P) {
  return flatten (P, var (P));
}

PRINT_SUGAR(TMPL,Polynomial)

#undef TMPL_DEF
#undef TMPL
#undef Polynomial
} // namespace mmx
#endif // __MMX_POLYNOMIAL_HPP
