
/******************************************************************************
* MODULE     : fft_bench.cpp
* DESCRIPTION: Bench FFT
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "fft.hpp"
#include "basix_utils.hpp"
//#include "numeric_extra.hpp"
using namespace mmx;

#define K modular<FFTP>
//#define K complexify<double>

template<typename C> C*
make (nat n) {
  C* a= new C[n];
  for (nat i=0; i<n; i++)
    a[i]= i*i;
  return a;
}

void
bench_fft (nat n) {
  fft_transformer<K> ffter (n);
  K* a= make<K> (n);
  nat count= 0;
  nat start= mmx_user_time ();
  while (mmx_user_time () < start + 2000) {
    ffter.direct_transform (a);
    count++;
  }
  double total= mmx_user_time () - start;
  cout.width (10); cout << n << " : ";
  cout.width (18); cout << (total/(count*n)) << " ms ";
  cout.width (18); cout << (total/count) << " ms\n";
  cout.flush();
  delete[] a;
}

int
main () {
  for (nat i=2; i<20; i++)
    bench_fft (1<<i);
  return 0;
}

