
/******************************************************************************
* MODULE     : relaxed_bench.cpp
* DESCRIPTION: Bench relaxed multiplication of series
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

//#include "floating.hpp"
//#include "numeric_extra.hpp"
#include "relaxed_series.hpp"
#include "basix_utils.hpp"

/******************************************************************************
* Test series
******************************************************************************/

namespace mmx {

template<typename C, typename V>
class test_series_rep: public series_rep<C,V> {
public:
  inline test_series_rep () {}
  generic expression (const generic& z) const {
    return generic ("test") (z); }
  C next () {
    if (this->n == 0) return C(0);
    return C(1) / C(this->n+1); }
};

template<typename C, typename V> series<C,V>
test_series () {
  return (series_rep<C,V>*) new test_series_rep<C,V> ();
}

template<typename C,typename V>
class sample_series_rep : public implicit_series_rep<C,V> {
public:
  sample_series_rep () {}
  generic expression (const generic& z) const {
    return generic ("sample") (z); }
  series<C,V> initialize () {
    this->initial (0)= 1;
    //series<C,V> p= relax_mul (this->me (), this->me (), 1<<16, 1<<16, true);
    //series<C,V> p= this->me () * this->me ();
    //series<C,V> p= this->me () * test_series<C,V> ();
    //series<C,V> p= test_series<C,V> () * test_series<C,V> ();
    //return shift_exp (p, 1);
    //return exp_op::def (this->me (), test_series<C,V> ());
    series<C,V > s1= this->me ();
    series<C,V > s2= derive (test_series<C,V> ());
    //series<C,V > s2= 3 * test_series<C,V> ();
    //series<C,V > s3= relax_mul (s1, s2, 1<<16, 0, true);
    series<C,V > s3= s1*s2;
    return integrate (s3);
    //return integrate (this->me () * derive (test_series<C,V> ()));
  }
};

template<typename C, typename V> series<C,V>
sample_series () {
  series_rep<C,V>* rep= new sample_series_rep<C,V> ();
  return implicit (series<C,V> (rep));
}

} // end namespace mmx

/******************************************************************************
* Benching
******************************************************************************/

using namespace mmx;

#define C modular<FFTP >
//#define C complexify<floating<> >
//#define C complex_double
//#define V naive_multiply
#define V relaxed<naive_multiply>

void
bench_relaxed (nat n) {
  //set_precision (256);
  //cout << determine_sizes (n) << " ";
  //cout << "\a\n";

  series<C,V > z= series<C,V > (1, 1); (void) z[n];
  series<C,V > f= test_series<C,V > (); (void) f[n];
  nat tsz= 2*n;
  vector<C> v (C(1), tsz);
  nat count, start;
  double total, t1= 0, t2= 0;

  count= 0;
  start= mmx_user_time ();
  while (mmx_user_time () < start + 1000 /*2000*/) {
    //series<C,V > g= f*f;
    //series<C,V > g= 1/(1+f);
    series<C,V > g= exp (f);
    //series<C,V > g= sample_series<C,V > ();
    //fft_count= mul_count= 0;
    (void) g[n-2];
    count++;
    //if (count == 1) break;
  }
  //cout << "{" << fft_count << "; " << mul_count << "}"; cout.flush ();
  total= mmx_user_time () - start;
  t1= total/count;
  cout << "*"; cout.flush ();

  count= 0;
  start= mmx_user_time ();
  while (mmx_user_time () < start + 1000) {
    //fft_count= 0;
    for (nat i=0; i<tsz; i++) v(i)= C(i);
    fft (seg (v), 2*n, false);
    fft (seg (v), 2*n, false);
    segment_mul (seg (v), seg (v), seg (v), 2*n);
    fft (seg (v), 2*n, true);
    count++;
  }
  //cout << "<" << fft_count << ">"; cout.flush ();
  total= mmx_user_time () - start;
  t2= total/count;

  cout.width (10); cout << n << " : ";
  cout.width (18); cout << t1 << " ms ";
  cout.width (18); cout << t2 << " ms ";
  cout.width (12); cout << t1/t2 << "\n";
  for (nat i=0; i<2000000000; i++);
}

int
main () {
  for (nat i=6; i<=24; i++)
    bench_relaxed (1<<i);
  return 0;
}
