
/******************************************************************************
* MODULE     : complexify_ball.hpp
* DESCRIPTION: Complexify_Balls
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_COMPLEXIFY_BALL_HPP
#define __MMX_COMPLEXIFY_BALL_HPP
#include "ball.hpp"
#include "complexify.hpp"
namespace mmx {
#define TMPL_DEF template<typename C,typename R,typename V=lipschitz_ball>
#define TMPL template<typename C,typename R,typename V>
#define Real_ball ball<C,R,V>
#define Ball ball<complexify<C>,R,V>

STYPE_TO_TYPE(TMPL,real_type,Ball,Real_ball);
STYPE_TO_TYPE(TMPL,abs_type,Ball,Real_ball);

TMPL inline int var_sign (const Ball& z) {
  if (z == 0) return 0; else return 1; }

/******************************************************************************
* Complex balls in Gaussian and polar representations
******************************************************************************/

TMPL Ball
gaussian (const Real_ball& re, const Real_ball& im= 0) {
  return Ball (gaussian (center (re), center (im)),
	       hypot_U (radius (re), radius (im)));
}

TMPL Ball
polar (const Real_ball& r, const Real_ball& t) {
  return gaussian (r * cos (t), r * sin (t));
}

TMPL Real_ball
Re (const Ball& z) {
  return Real_ball (Re (center (z)), radius (z), false);
}

TMPL Real_ball
Im (const Ball& z) {
  return Real_ball (Im (center (z)), radius (z), false);
}

TMPL Real_ball
abs (const Ball& z) {
  if (is_infinity (z)) return Plus_infinity (Real_ball);
  return Real_ball (abs (center (z)), radius (z));
}

TMPL Real_ball
arg (const Ball& z) {
  return atan2 (Im (z), Re (z));
}

/******************************************************************************
* Further basic functions
******************************************************************************/

TMPL generic
flatten (const Ball& z) {
  if (is_nan (z)) return Nan (generic);
  if (is_fuzz (z)) return Fuzz (generic);
  if (is_infinity (z)) return Infinity (generic);
  if (is_exact_zero (z)) return 0;
  if (Re (z) != 0 && Im (z) == 0)
    return flatten (Re (z));
  if (Re (z) == 0 && Im (z) != 0)
    return flatten (Im (z)) * Imaginary (generic);
  return flatten (Re (z)) + flatten (Im (z)) * Imaginary (generic);
}

TMPL generic
flatten (const Ball& z, nat dd) {
  if (is_nan (z)) return Nan (generic);
  if (is_fuzz (z)) return Fuzz (generic);
  if (is_infinity (z)) return Infinity (generic);
  if (is_exact_zero (z)) return 0;
  if (Re (z) != 0 && Im (z) == 0)
    return flatten (Re (z), dd);
  if (Re (z) == 0 && Im (z) != 0)
    return flatten (Im (z), dd) * Imaginary (generic);
  return flatten (Re (z), dd) + flatten (Im (z), dd) * Imaginary (generic);
}

TMPL Ball
conj (const Ball& z) {
  return Ball (gaussian (Re (center (z)), -Im (center (z))),
	       radius (z), false);
}

TMPL Ball
times_i (const Ball& z) {
  return Ball (gaussian (-Im (center (z)), Re (center (z))),
	       radius (z), false);
}

TMPL Ball
over_i (const Ball& z) {
  return Ball (gaussian (Im (center (z)), -Re (center (z))),
	       radius (z), false);
}

/******************************************************************************
* Elementary functions
******************************************************************************/

TMPL Ball
sqrt (const Ball& z) {
  if (is_zero (z)) return Fuzz (Ball);
  return Ball (sqrt (center (z)),
	       div_U (radius (z) >> 1, sqrt_D (abs_D (z))));
}

TMPL Ball
exp (const Ball& z) {
  return Ball (exp (center (z)),
	       mul_U (exp_U (bnd_U (Re (z))), radius (z)));
}

TMPL Ball
log (const Ball& z) {
  if (is_zero (z)) return Nan (Ball);
  R rz= radius (z);
  complexify<C> cz= center (z);
  return Ball (log (center (z)),
	       div_U (radius (z), abs_D (z)));
}

TMPL Ball
cos (const Ball& z) {
  return Ball (cos (center (z)),
	       mul_U (sinh_U (bnd_U (Im (z))), radius (z)));
}

TMPL Ball
sin (const Ball& z) {
  return Ball (sin (center (z)),
	       mul_U (cosh_U (bnd_U (Im (z))), radius (z)));
}

TMPL inline Ball
cosh (const Ball& z) {
  return Ball (cosh (center (z)),
	       mul_U (sinh_U (bnd_U (abs (Re (z)))), radius (z)));
}

TMPL inline Ball
sinh (const Ball& z) {
  return Ball (sinh (center (z)),
	       mul_U (cosh_U (bnd_U (abs (Re (z)))), radius (z)));
}

TMPL Ball
acos (const Ball& z) {
  return over_i (acosh (z));
}

TMPL Ball
asin (const Ball& z) {
  return over_i (asinh (times_i (z)));
}

TMPL Ball
atan (const Ball& z) {
  return over_i (atanh (times_i (z)));
}

#undef TMPL_DEF
#undef TMPL
#undef Real_ball
#undef Ball
} // namespace mmx
#endif // __MMX_COMPLEXIFY_BALL_HPP
