
/******************************************************************************
* MODULE     : real_ball.hpp
* DESCRIPTION: Standard real balls
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_REAL_BALL_HPP
#define __MMX_REAL_BALL_HPP
#include "rounded_floating.hpp"
#include "type_props.hpp"
#include "wrap.hpp"
namespace mmx {
typedef floating<> real_ball_center;
typedef floating<single_floating> real_ball_radius;

class real_ball {
MMX_ALLOCATORS;
private:
  // Define fields of same size as "Ball" type (see implementation class)
  // IMPORTANT: The sizes MUST match for binary compatibility
  void* c;
  void* r;

public:
  real_ball (const real_ball& x);
  ~real_ball ();
  real_ball& operator = (const real_ball& x);
  real_ball ();
  real_ball (int i);
  real_ball (nat i);
  real_ball (double x);
  real_ball (const integer& i);
  real_ball (const rational& x);
  real_ball (const real_ball_center& x);
  real_ball (const real_ball_center& c, const real_ball_radius& r);
};

STYPE_TO_TYPE(STMPL,center_type,real_ball,real_ball_center);
STYPE_TO_TYPE(STMPL,radius_type,real_ball,real_ball_radius);

// Definitions as plain routines fixes compilation problem with gcc 2.95.3
real_ball real_ball_nan ();
real_ball real_ball_plus_infinity ();
real_ball real_ball_minus_infinity ();
real_ball real_ball_infinity ();
real_ball real_ball_fuzz ();
real_ball real_ball_log2 ();
real_ball real_ball_pi ();
real_ball real_ball_euler ();

STMPL
struct constant_helper<real_ball > {
  inline static real_ball nan () { return real_ball_nan (); }
  inline static real_ball plus_infinity () {
    return real_ball_plus_infinity (); }
  inline static real_ball minus_infinity () {
    return real_ball_minus_infinity (); }
  inline static real_ball infinity () { return real_ball_infinity (); }
  inline static real_ball fuzz () { return real_ball_fuzz (); }
  inline static real_ball log2 () { return real_ball_log2 (); }
  inline static real_ball pi () { return real_ball_pi (); }
  inline static real_ball euler () { return real_ball_euler (); }
};

STMPL
struct make_ball_helper<real_ball,real_ball_center,real_ball_radius> {
  static inline real_ball
  val (const real_ball_center& c, const real_ball_radius& r) {
    return real_ball (c, r); }
};

WRAP_BASIC_PROTO(,real_ball);
WRAP_FIELD_PROTO(,real_ball);
WRAP_SHIFT_PROTO(,real_ball);
WRAP_RING_COMPARE_PROTO(,real_ball);
WRAP_NUMBER_CHECK_PROTO(,real_ball);
WRAP_ELEMENTARY_PROTO(,real_ball);
WRAP_REAL_PROTO(,real_ball);

generic flatten (const real_ball& x, nat dd);
real_ball_center center (const real_ball& x);
real_ball_radius radius (const real_ball& x);
real_ball_center lower (const real_ball& x);
real_ball_center upper (const real_ball& x);
real_ball_radius abs_D (const real_ball& x);
real_ball_radius abs_U (const real_ball& x);
real_ball_radius bnd_D (const real_ball& x);
real_ball_radius bnd_U (const real_ball& x);

} // namespace mmx
#endif // __MMX_REAL_BALL_HPP
