
/******************************************************************************
* MODULE     : exception.hpp
* DESCRIPTION: The type for exceptional behaviour
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __EXCEPTION_HPP
#define __EXCEPTION_HPP
#include "string.hpp"

namespace mmx {

class exception {
  generic rep;
public:
  inline exception (const generic& g): rep (g) {}
  inline exception (const string& msg, const generic& where):
    rep (gen ("runtime error", msg, where)) {}
  inline exception (const string& msg, const generic& a, const generic& where):
    rep (gen ("runtime error", msg, a, where)) {}
  inline exception (const exception& t): rep (t.rep) {}
  inline friend bool hard_eq (const exception& e1, const exception& e2) {
    return hard_eq (e1.rep, e2.rep); }
  inline friend bool hard_neq (const exception& e1, const exception& e2) {
    return hard_neq (e1.rep, e2.rep); }
  inline friend bool eq (const exception& e1, const exception& e2) {
    return eq (e1.rep, e2.rep); }
  inline friend bool neq (const exception& e1, const exception& e2) {
    return neq (e1.rep, e2.rep); }
  inline bool operator == (const exception& e) const {
    return eq (rep, e.rep); }
  inline bool operator != (const exception& e) const {
    return neq (rep, e.rep); }
  inline generic operator * () const { return rep; }
};

inline nat hash (const exception& t) { return hash (*t); }

inline generic flatten (const exception& t) { return flatten (*t); }

PRINT_SUGAR(,exception)

/******************************************************************************
* Special exceptions
******************************************************************************/

generic wrong_nr_args (const generic& where);
generic type_mismatch (const generic& expect, const generic& where);

} // namespace mmx
#endif // __EXCEPTION_HPP
