
/******************************************************************************
* MODULE     : compound.hpp
* DESCRIPTION: Compound generic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __COMPOUND_HPP
#define __COMPOUND_HPP
#include "array.hpp"
#include "mixed_symbol.hpp"
namespace mmx {

/******************************************************************************
* Compound expressions
******************************************************************************/

class compound {
  array<generic> v;
public:
  inline compound (const array<generic>& v2): v (v2) {}
  inline compound (const compound& v2): v (v2.v) {}
  inline friend bool hard_eq (const compound& v1, const compound& v2) {
    return hard_eq (v1.v, v2.v); }
  inline friend bool hard_neq (const compound& v1, const compound& v2) {
    return hard_neq (v1.v, v2.v); }
  inline friend bool eq (const compound& v1, const compound& v2) {
    return eq (v1.v, v2.v); }
  inline friend bool neq (const compound& v1, const compound& v2) {
    return neq (v1.v, v2.v); }
  inline bool operator == (const compound& v2) const { return v == v2.v; }
  inline bool operator != (const compound& v2) const { return v != v2.v; }
  inline array<generic> operator * () const { return v; }
  inline friend nat hash (const compound& v) { return hash (*v); }
};

array<generic> compound_to_array (const generic& g);
generic array_to_compound (const array<generic>& v);
generic flatten (const compound& c);

inline compound melt (const compound& c) { return compound (melt (*c)); }
inline compound fuse (const compound& c) { return compound (fuse (*c)); }

/******************************************************************************
* Compound expressions as generics
******************************************************************************/

STMPL
struct internal<compound> {
  typedef mixed_symbol<compound> Repr;
  static inline Repr encode (const compound& x) { return Repr (x); }
  static inline Repr encode (const compound& x, bool flag) {
    return Repr (x, flag); }
  static inline compound decode (const Repr& x) { return *x; }
};

STMPL
struct inspector<compound> {
  static inline nat dim (const compound& v) { return mmx::N(*v); }
  static inline generic access (const compound& v, nat i) {
    VERIFY_DOMAIN (i < mmx::N(*v));
    return (*v)[i];
  }
};

} // namespace mmx
#endif // __COMPOUND_HPP
