
/******************************************************************************
* MODULE     : lisp_syntax.hpp
* DESCRIPTION: Parsing and printing expressions in lisp form
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __SYNTAX_HPP
#define __SYNTAX_HPP
// NOTE: lisp_syntax.hpp is included by basix.hpp
namespace mmx {

/******************************************************************************
* Lisp input and output
******************************************************************************/

void lisp_print (ostream& out, const generic& g);
generic from_lisp (const string& s);

template<typename C> inline ostream&
print_lisp (ostream& out, const C& x) {
  lisp_print (out, flatten (x));
  return out;
}

/******************************************************************************
* Flattening
******************************************************************************/

template<typename C> string print_to_string (const C& x);
template<typename C> string print_to_lisp (const C& x);
extern void* flatten_aux_ptr;
extern string as_string (void* ptr);

template<typename C> generic
flatten (const C& x) {
  if (flatten_aux_ptr == (void*) &x)
    return print_to_string (x);
  else {
    void* old= flatten_aux_ptr;
    flatten_aux_ptr= (void*) &x;
    generic r= from_lisp (print_to_lisp (x));
    flatten_aux_ptr= old;
    return r;
  }
}

template<typename C> inline generic
flatten (C* const & x) {
  return as_string ((void*) x);
}

generic flatten (bool b);
generic flatten (int i);
generic flatten (nat i);
generic flatten (float x);
generic flatten (double x);
generic flatten (char* s);
generic flatten (const string& s);

/******************************************************************************
* Sugar
******************************************************************************/

extern void select_output_evaluator ();
extern void restore_evaluator ();
extern bool set_frac_flag (bool new_val);

#define PRINT_SUGAR(TMPL,T)				        \
TMPL inline ostream& operator << (ostream& out, const T& x) {	\
  select_output_evaluator ();                                   \
  out << flatten (x);                                           \
  restore_evaluator ();                                         \
  return out;                                                   \
}

// IMPORTANT NOTE: the flattening routines should only involve trivial
// operations on generics which are defined in the output evaluator.
// Indeed, certain operations like < are not defined in the output evaluator.


} // namespace mmx
#endif // __SYNTAX_HPP
