AC_DEFUN([AC_MMX_OPTION_DEBUG], [
    if test "$CXXFLAGS" = "-g -O2"; then
        CXXFLAGS=""
    fi

    AC_ARG_ENABLE(debug,
    AC_HELP_STRING(
        [--enable-debug],
        [install a debugging enable executable [[-ggdb]]]),
    [], [enable_debug="no"])

    if test "$enable_debug" = "yes"; then
        if test "$GXX" = "yes"; then
            enable_debug="-ggdb"
        else
            enable_debug="-g3"
        fi
    fi

    AH_TEMPLATE([ENABLE_DEBUG],[Compile with debugging information and checks])

    if test "$enable_debug" = "no"; then
        AC_MSG_RESULT([disabling debugging])
    else
        AC_MSG_RESULT([enabling debugging, $enable_debug])
        AC_DEFINE([ENABLE_DEBUG],[])
        CXXFLAGS="$CXXFLAGS $enable_debug"
    fi
])

AC_DEFUN([AC_MMX_OPTION_OPTIMIZE], [
    if test "$CXXFLAGS" = "-g -O2"; then
        CXXFLAGS=""
    fi

    AC_ARG_ENABLE(optimize,
    AC_HELP_STRING(
        [--enable-optimize],
        [compile with optimizations [[guessed]]]),
    [], [enable_optimize="yes"])

    case "$enable_optimize" in
        yes)
	    CXXFLAGS="$CXXFLAGS -ansi -Wall -O3 -fexpensive-optimizations"
	    AC_MSG_RESULT([enabling optimizations, $CXXFLAGS]) ;;
	no)
	    AC_MSG_RESULT([using standard optimization flags, $CXXFLAGS]) ;;
	*)
	    CXXFLAGS="$CXXFLAGS $enable_optimize"
	    AC_MSG_RESULT([customizing optimizations, $CXXFLAGS]) ;;
    esac

    CXXFLAGS="$CXXFLAGS -ftemplate-depth-25"
])

AC_DEFUN([AC_MMX_OPTION_EXCEPTIONS], [
    AC_ARG_ENABLE(exceptions,
    AC_HELP_STRING(
        [--enable-exceptions],
        [use C++ exception handling mechanism [[yes]]]),
    [], [enable_exceptions="yes"])

    AH_TEMPLATE([ENABLE_EXCEPTIONS],[Use C++ exception mechanism])

    if test "$enable_exceptions" = "no"; then
        AC_MSG_RESULT([disabling exception handling])
    else
        AC_MSG_RESULT([enabling exception handling])
        AC_DEFINE([ENABLE_EXCEPTIONS],[])
    fi
])

AC_DEFUN([AC_MMX_OPTION_VERIFY], [
    AC_ARG_ENABLE(verify,
    AC_HELP_STRING(
        [--enable-verify],
        [verify correctness of low-level operations [[no]]]),
    [], [enable_verify="no"])

    AH_TEMPLATE([ENABLE_VERIFY],[Verify correctness of low-level operations])

    if test "$enable_verify" = "no"; then
        AC_MSG_RESULT([disabling correctness checks for low-level operations])
    else
        AC_MSG_RESULT([enabling correctness checks for low-level operations])
        AC_DEFINE([ENABLE_VERIFY],[])
    fi
])

AC_DEFUN([AC_MMX_OPTION_GLUE], [
    AC_ARG_ENABLE(glue,
    AC_HELP_STRING(
	[--enable-glue],
	[compile glue for interpreter [[yes]]]),
    [], [enable_glue="yes"])

    AM_CONDITIONAL([GLUE_OPT], [test "$enable_glue" = yes])

    if test "$enable_glue" = "no"; then
        AC_MSG_RESULT([disabling glue compilation for interpreter])
    else
        AC_MSG_RESULT([enabling glue compilation for interpreter])
    fi
])

AC_DEFUN([AC_MMX_OPTION_TEST], [
    AC_ARG_ENABLE(test,
    AC_HELP_STRING(
	[--enable-test],
	[compile additional test programs [[no]]]),
    [], [enable_test="no"])

    AM_CONDITIONAL([TEST_OPT], [test "$enable_test" = yes])

    if test "$enable_test" = "no"; then
        AC_MSG_RESULT([disabling compilation of additional test programs])
    else
        AC_MSG_RESULT([enabling compilation of additional test programs])
    fi
])

AC_DEFUN([AC_MMX_OPTION_BENCH], [
    AC_ARG_ENABLE(bench,
    AC_HELP_STRING(
	[--enable-bench],
	[compile additional benchmark programs [[no]]]),
    [], [enable_bench="no"])

    AM_CONDITIONAL([BENCH_OPT], [test "$enable_bench" = yes])

    if test "$enable_bench" = "no"; then
        AC_MSG_RESULT([disabling compilation of additional benchmark programs])
    else
        AC_MSG_RESULT([enabling compilation of additional benchmark programs])
    fi
])

AC_DEFUN([AC_MMX_MODULE], [
    AC_MMX_OPTION_DEBUG
    AC_MMX_OPTION_OPTIMIZE
    AC_MMX_OPTION_EXCEPTIONS
    AC_MMX_OPTION_VERIFY
    AC_MMX_OPTION_GLUE
    AC_MMX_OPTION_TEST
    AC_MMX_OPTION_BENCH
])
