/******************************************************************************
* MODULE     : input.cpp
* DESCRIPTION: mathemagix shell input and completion functions
* COPYRIGHT  : (C) 2007  Gregoire Lecerf
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <stdio.h>
#include <basix.hpp>
#ifdef MMX_SUPPORTS_READLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif // MMX_SUPPORTS_READLINE

#include <list.hpp>
#include <string.hpp>
#include <environment.hpp>
#include <evaluator.hpp>
#include <exception.hpp>
#include <shell.hpp>
#include <math_syntax.hpp>
#include <parse_tools.hpp>

namespace mmx {
using std::cout;
using std::cin;

  /*
   * Normal output
   */

void
shell_terminal_output (const generic& g, const generic& t) {
  if (math_mode) {
    print_math(cout, g);
  }
  else
    cout << g;
  if (neq (t, "None")) cout << ": " << t;
  cout << "\n";
}

  /*
   * Error output
   */

static void
print_underlined_source (const nat& n,
			 const source_position& b,
			 const source_position& e) {
  // for interactive sources in terminal only
  nat j;
  string line;
  
  line = get_source (n, b.line);
  cout << line << "\n";
  for (j=0; j < b.column; ++j)
    cout << " ";
  if (b.line == e.line) {
    for (; j < e.column; ++j)
      cout << "~";
  }
  else {
    if (e.line == b.line+1 && e.column == 0)
      cout << "^";
    else {
      for (; j < N(line); ++j)
	cout << "~";
      cout << "\n";
      if (e.column == 0) {
	if (e.line - 1 > b.line + 1)
	  cout << "...\n";
	line = get_source (n, e.line-1);
	cout << line << "\n";
	for (j=0; j < N(line); ++j)
	  cout << "~";
	if (N(line) == 0)
	  cout << "^";
      }
      else {
	if (e.line > b.line + 1)
	  cout << "...\n";
	line = get_source (n, e.line);
	cout << line << "\n";
	for (j=0; j < e.column; ++j)
	  cout << "~";
      }
    }
  }
  cout << "\n";
}

bool 
shell_terminal_print_syntax_errors (const list<syntax_error>& errors) {
  bool raise_error = false;
  iterator<syntax_error> it = iterate(errors);
  string line;
  
  for (; busy(it); ++it) {
    if ((*it).type == "error") raise_error = true;
    if ((*it).file_name != "")
      cout << (*it).file_name << ":";
    //else
    //  cout << "input " << (*it).input_number + 1 << ":";
    cout << (*it).begin.line + 1 << ":"
	 << (*it).begin.column + 1 << ": ";
    if ((*it).type != "error")
      cout << (*it).type << ", ";
    cout << (*it).message << "\n";
    if ((*it).file_name == "")
      print_underlined_source ((*it).input_number,
			       (*it).begin, (*it).end);
  }
  return raise_error;
}
  
bool
shell_terminal_print_exception (const generic& g) {
  nat i;
  string s;
  generic err;
  source_location l;

  //  assert (is<exception> g);
  err = *as <exception> (g);
  assert (is <compound> (err));
  assert (N(err) > 2);
  source_locate(err[N(err)-1], l);
  if (nil (l)) cout << "Inside " << err[N(err)-1] << ": ";
  else {
    cout << l.begin.line+1 << ":" 
	 << l.begin.column+1 << ": ";
  }
  //assert (is <string> err[0]);
  //assert (is <string> err[1]);
  cout << err[0] << ", ";
  cout << err[1] << " ";
  for (i = 2; i+1 < N(err); ++i)
    cout << err[i] << " ";
  cout << "\n";
  if (!nil (l) && l.file_name == "")
    print_underlined_source (l.input_number, l.begin, l.end);
  return true;
}
  
} // namespace mmx
