
/******************************************************************************
* MODULE     : linear_map.hpp
* DESCRIPTION: Abstract linear operators on generic expressions
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __LINEAR_MAP_HPP
#define __LINEAR_MAP_HPP
#include "basix.hpp"
namespace mmx {

#define MAP_MULTIPLICATION  1
#define MAP_DERIVATION      2
#define MAP_DIFFERENCE      4
#define MAP_SUBSTITUTION    8

class linear_map;
class linear_map_rep: public rep_struct {
protected:
  nat map_type;
public:
  inline linear_map_rep (nat map_type2): map_type (map_type2) {}
  inline ~linear_map_rep () {}
  inline bool is_multiplication () {
    return (map_type & MAP_MULTIPLICATION) != 0; }
  inline bool is_derivation () {
    return (map_type & MAP_DERIVATION) != 0; }
  inline bool is_difference () {
    return (map_type & MAP_DIFFERENCE) != 0; }
  inline bool is_substitution () {
    return (map_type & MAP_SUBSTITUTION) != 0; }
  virtual generic get_hash_value () const;
  virtual generic expression () const = 0;
  virtual generic apply (const generic& g) const = 0;
};

class linear_map {
INDIRECT_PROTO (linear_map, linear_map_rep)
  inline bool operator == (const linear_map& op) const {
    return rep == op.rep; }
  inline bool operator != (const linear_map& op) const {
    return rep != op.rep; }
  inline generic operator () (const generic& x) {
    return rep->apply (x); }
  friend inline nat hash (const linear_map& op) const {
    return op->get_hash_value (); }
  friend inline generic flatten (const linear_map& op) {
    return op->expression (); }
};
INDIRECT_IMPL (linear_map, linear_map_rep)
PRINT_SUGAR(,linear_map);

linear_map left_multiplication (const generic& x);
linear_map partial_derivation (const generic& x);
linear_map post_composition (const generic& which, const generic& by);

} // namespace mmx
#endif // __LINEAR_MAP_HPP
