
/******************************************************************************
* MODULE     : symbolic_test.cc
* DESCRIPTION: Test symbolic expressions
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rational.hpp"
#include "symbolic.hpp"
using namespace mmx;

#define Symbolic symbolic<rational>

int
main () {
  Symbolic f ("f");
  Symbolic x ("x");
  Symbolic y ("y");
  Symbolic z= f (x, y);
  Symbolic p (49);

  cout << "x\t= " << x << "\n";
  cout << "y\t= " << y << "\n";
  cout << "z\t= " << z << "\n";
  cout << "p\t= " << p << "\n\n";

  cout << "x = y\t= " << (x==y) << "\n";
  cout << "x != y\t= " << (x!=y) << "\n";
  cout << "z = z\t= " << (z==z) << "\n";
  cout << "x = z\t= " << (x==z) << "\n";
  cout << "x y\t= " << (x*y) << "\n";
  cout << "x + 1\t= " << (x+1) << "\n";
  cout << "x + 0\t= " << (x+0) << "\n";
  cout << "z z\t= " << z*z << "\n";
  cout << "z z z\t= " << z*z*z << "\n";
  cout << "z / 7\t= " << z/7 << "\n";
  cout << "z / 111\t= " << z/111 << "\n";
  cout << "p p\t= " << p*p << "\n";
  cout << "p / p\t= " << p/p << "\n";
  cout << "21 / p\t= " << 21/p << "\n\n";

  cout << "x x\t\t\t= " << x*x << "\n";
  cout << "2 x y\t\t\t= " << 2*x*y << "\n";
  cout << "x 2 y\t\t\t= " << x*2*y << "\n";
  cout << "2 x y + y x\t\t= " << 2*x*y + y*x << "\n";
  cout << "2 x y + x y\t\t= " << 2*x*y + x*y << "\n";
  cout << "2 x y - y x\t\t= " << 2*x*y - y*x << "\n";
  cout << "2 x y - y x - x y\t= " << 2*x*y - y*x - x*y << "\n";
  cout << "x (x + y)\t\t= " << x*(x+y) << "\n";
  cout << "(x + y) (x - y)\t\t= " << (x+y) * (x-y) << "\n";
  cout << "2 (x + y) - x\t\t= " << 2*(x+y) - x << "\n\n";

  cout << "x x / x\t\t\t= " << x*x/x << "\n";
  cout << "x / (2 y)\t\t= " << x/(2*y) << "\n";
  cout << "x / (2 y) x\t\t= " << (x/(2*y))*x << "\n";
  cout << "x / (2 y) 2 x\t\t= " << (x/(2*y))*(2*x) << "\n";
  cout << "x / (2 y) 4 x\t\t= " << (x/(2*y))*(4*x) << "\n";
  cout << "(x^2)^3\t\t\t= " << pow (pow (x, 2), 3) << "\n";
  cout << "(x^x)^y\t\t\t= " << pow (pow (x, x), y) << "\n";
  cout << "(x y)^y\t\t\t= " << pow (x*y, y) << "\n";
  cout << "(x y)^y / x\t\t= " << pow (x*y, y) / x << "\n";
  cout << "(x y)^y / (x y)\t\t= " << pow (x*y, y) / (x*y) << "\n";
  cout << "(x y)^y / (x y)^2\t= " << pow (x*y, y) / pow (x*y, 2) << "\n";
  cout << "((x + y)^x)^y\t\t= " << pow (pow (x+y, x), y) << "\n";
  cout << "2^x 2^(1-x)\t\t= " << pow (2, x) * pow (2, 1-x) << "\n";
  cout << "3 2^x 2^(1-x)\t\t= " << 3 * pow (2, x) * pow (2, 1-x) << "\n";
  cout << "(x y)^3 / (y z)^5\t= " << pow (x*y, 3) / pow (y*z, 5) << "\n";
  cout << "(x y)^5 / (y z)^3\t= " << pow (x*y, 5) / pow (y*z, 3) << "\n\n";

  cout << "exp (x)\t\t\t= " << exp (x) << "\n";
  cout << "exp (x) exp (y)\t\t= " << exp (x) * exp (y) << "\n";
  cout << "log (2 x y)\t\t= " << log (2*x*y) << "\n";
  cout << "log (2 x / y)\t\t= " << log (2*x/y) << "\n";
  cout << "sqrt (x)\t\t= " << sqrt (x) << "\n";
  cout << "sqrt (x)'\t\t= " << derive (sqrt (x), x) << "\n";
  cout << "sqrt (x)''\t\t= " << derive (sqrt (x), x, 2) << "\n";
  cout << "(x^x)'\t\t\t= " << derive (pow (x, x), x) << "\n";
  cout << "(x^x)''\t\t\t= " << derive (pow (x, x), x, 2) << "\n";
  cout << "(x^(x^x))'\t\t= " << derive (pow (x, pow (x, x)), x) << "\n";
  cout << "((x y) / (y x))'\t= " << derive ((x*y) / (y*x), x) << "\n";
  cout << "((x y) / (y x))''\t= " << derive ((x*y) / (y*x), x, 2) << "\n";
  cout << "(x y^y / x)'\t\t= " << derive ((x * pow (y, y)) / x, x) << "\n";
  cout << "f (x)'\t\t\t= " << derive (f(x), x) << "\n";
  cout << "f (x sin (x))'\t\t= " << derive (f (x * sin(x)), x) << "\n";
  cout << "f (x, sin (x y), y)'\t= " << derive (f (x, sin(x*y), y), x) << "\n";
  cout << "tan (tan (x))'\t\t= " << derive (tan (tan (x)), x) << "\n";
  cout << "tan (tan (x))''\t\t= " << derive (tan (tan (x)), x, 2) << "\n";
  cout << "arcsin (arcsin (x))'\t= " << derive (asin (asin (x)), x) << "\n";
  cout << "arcsin (x + y)'\t\t= " << derive (asin (x + y), x) << "\n";
  //cout << "f(x*sin(x))''\t\t= "
  //<< derive (derive (f (x * sin(x)), x), x) << "\n";
  //cout << "(x^(x^x))''\t\t= "
  //<< derive (derive (pow (x, pow (x, x)), x), x) << "\n";

  return 0;
}
